const { InputManager } = require ('./input_manager')
const { ResourceManager } = require('./resource_manager')
const { PlaylistsManager } = require('./playlists_manager')
const { Player } = require('./player')
const config = require('../config.json')

class GuildsManager {
  #guilds

  constructor() {
    this.#guilds = {}
  }

  toString() {
    return "GuildsManager#" + Object.keys(this.#guilds).map(g => `[guid:${g}]`)
  }

  get_manager(guild, name) {
    if (guild?.guild?.id) guild = guild.guild.id // we may receive a message instead of a guid.id
    return this.#guilds[guild][name];
  }

  async handle_message(message) {
    const guild_id = message.guild.id
    if (!this.#guilds[guild_id]) {
      const current_config = config[message.guild.id] || config['default']
      this.#guilds[guild_id] = {
        input: new InputManager(this, current_config),
        player: new Player(this),
        resource: new ResourceManager(this),
        playlists: new PlaylistsManager(this),
      }
      await Promise.all(Object.values(this.#guilds[guild_id]).map(async (manager) => {
        if (manager.init) await manager.init()
      }))
    }
    const guild_action = await this.#guilds[guild_id].input.handle_message(message)
    if (guild_action) {
      if (!this.#guilds[guild_id]?.[guild_action.target]?.[guild_action.action]) {
        message.channel.send(`The command ${guild_action.target} ${guild_action.action} is not implemented yet`)
      } else {
        const output = await this.#guilds[guild_id][guild_action.target][guild_action.action](guild_action)
        return output
      }
    }
  }
}

module.exports = { GuildsManager }
