const { Playlist } = require('./playlist')
const { GuildAction } = require('./guild_action')

/**
 * @typedef {Object} Trigger
 * @property {RegExp} pattern The pattern that will trigger the trigger
 * @property {string} target The target class
 * @property {string} action The method to call on trigger
 * @property {string} [ttl=1 hour] The time to live (or timeout) of the trigger
 * @property {boolean} [user_shared=false] <code>true</code> if anyone can trigger the action, <code>false</code> otherwise
 * @property {number} [repeat=1] How many times the trigger should be called
 */

/**
 * @class
 */
class InputManager {
  #config
  #triggers
  #gmanager

  constructor(gmanager, config) {
    this.#config = config
    this.#gmanager = gmanager
    this.#triggers = []
  }

  async handle_message(message) {
    if (message.content.slice(0, this.#config.prefix.length) === this.#config.prefix) {
      const [command, ...command_params] = message.content.slice(this.#config.prefix.length).trim().split(/\s+/)
      console.error("> command", command)
      const command_target_action = this.#config.commands[command]
      console.error("> command_target_action", command_target_action)
      if (command_target_action) {
        return new GuildAction({ ...command_target_action, message, params: command_params })
      }
    }

    const trigger_target_action_idx = this.#triggers.findIndex(t => t.pattern.test(message.content))
    // TODO: add user, etc check
    if (trigger_target_action_idx > -1) {
      const trigger_target_action = this.#triggers[trigger_target_action_idx]
      console.error("> trigger_target_action", trigger_target_action)
      trigger_target_action.repeat -= 1
      if (trigger_target_action.repeat <= 0) {
        delete this.#triggers[trigger_target_action_idx]
        this.#triggers = this.#triggers.filter(Boolean)
      }
      const params = message.content
            .split(trigger_target_action.pattern)
            .splice(1)[0]
            .trim()
            .split(/\s+/)
      return new GuildAction({ ...trigger_target_action, message, params: params })
    }
    return null
  }

  /**
   * @param {Trigger} trigger The trigger to add
   */
  add_trigger({ pattern, target, action, ttl = '1 hour', user_shared = false, repeat = 1 }) {
    this.#triggers.push({
      pattern,
      target,
      action,
      ttl,
      user_shared,
      repeat
    })
    // TODO add async loop to cleanup triggers by ttl
  }
}

module.exports = { InputManager }
