const ytdl  = require('ytdl-core')

const { Playlist } = require('./playlist');

class Player {
  #voice
  #dispatcher
  #playlist
  #gmanager
  #is_playing

  get_playlist() { return this.#playlist }
  set_playlist(playlist) {
    this.#playlist = playlist
    return this
  }

  constructor(gmanager) {
    this.#gmanager = gmanager
    this.#voice = null
    this.#dispatcher = null
    this.#playlist = new Playlist()
    this.#is_playing = false
  }

  // https://www.youtube.com/watch?v=Yom8nNqmxvQ
  async stream({ url }) {
    this.#is_playing = true
    const stream = ytdl(url, { filter: 'audioonly' })
    this.#dispatcher = this.#voice.play(stream)
    this.#dispatcher.on('finish', () => this.next())
    return this;
  }

  isConnected(channel) {}

  async connect(channel) {
    this.#voice = await channel.join()
    this.#voice.voice.setSelfDeaf(true)
  }

  async add({ message, params }) {
    if (!message.member?.voice?.channel) {
      message.channel.send('Not connected to voice')
      // cannot connect
    } else {
      await this.connect(message.member.voice.channel)
      if (params[0]) {
        const full_title = await this.#gmanager.get_manager(message.guild.id, 'resource').discover_youtube_full_title(params[0])
        this.#playlist.push({ url: params[0], full_title })
      }
      if (!this.#is_playing) {
        this.stream(this.#playlist.get_current_track())
      }
    }
  }

  /**
   * params[0] is the id of the track to unqueue
   * TODO ensure consistency with index/id displayed...
   */
  async remove({ message, params }) {
    const id_to_remove = Number(params[0])
    const index_to_remove = id_to_remove - 1;
    if (!isNaN(id_to_remove)) {
      const track = this.#playlist.get_track_by_index(index_to_remove)
      if (track && this.#playlist.delete(index_to_remove)) {
        message.channel.send(`:white_check_mark: The track ${id_to_remove} \`${track.full_title}\` has been removed`)
      } else {
        message.channel.send(`:warning: The track ${id_to_remove} has not been removed or found`)
      }
    }
  }

  async next() {
    const track = this.#playlist.next().get_current_track();
    if (track) {
      this.stream(track)
    } else {
      this.#is_playing = false;
    }
  }

  async prev() {
    const track = this.#playlist.previous().get_current_track();
    if (track) {
      this.stream(track)
    } else {
      this.#is_playing = false;
    }
  }

  async list({ message, params }) {
    if (this.#playlist.is_empty()) {
      message.channel.send("Empty playlist")
    } else {
      message.channel.send(
        "Current playlist:\n" +
          "```json\n" +
          this.#playlist.mapTracks((t, id, _, current) =>(current === t ? '(*) ' : '') + `${id + 1} ${t.full_title}`).join("\n") +
          "```\n",
      )
    }
  }
}

module.exports = { Player }
