class Playlist {
  #current_index
  #track_list

  constructor({ tracks = [], loop = true } = {}) {
    this.#current_index = 0
    this.#track_list = tracks
  }

  toJSON() {
    return this.#track_list
  }

  mapTracks(callback) {
    const current = this.get_current_track()
    return this.#track_list.map((t, idx, all) => callback(t, idx, all, current))
  }

  next() {
    this.#current_index = (this.#current_index + 1) % this.#track_list.length
    return this
  }

  previous() {
    this.#current_index = (this.#current_index - 1 + this.#track_list.length) % this.#track_list.length
    return this
  }

  get_current_track() {
    if (this.#current_index >= this.#track_list.length) return null
    return this.#track_list[this.#current_index]
  }

  get_track_by_index(track_index) {
    if (track_index >= this.#track_list.length) return null
    return this.#track_list[track_index]
  }

  push(track) {
    this.#track_list.push(track)
    return this
  }

  delete(track_index) {
    if (track_index >= this.#track_list.length) return false
    delete this.#track_list[track_index]
    this.#track_list = this.#track_list.filter(Boolean)
    return true
  }

  is_empty() {
    return this.#track_list.length === 0
  }

  toString() {
    return this.#track_list.map(t => t.full_title || t.title || t.url).join(', ')
  }
}

module.exports = { Playlist }
